#!/bin/bash

echo testing Glauber Ising
../src/shi.exe -n -ptest.in -ctestTraj.shi > test.out
echo test finished, first few diffs are:
diff -y testTraj.shi testTraj.shi.ref | head
diff -q testTraj.shi testTraj.shi.ref
echo; echo
 
echo testing Glauber Ising with shear flow
../src/shi.exe -n -ptestShear.in -ctestShearTraj.shi > testShear.out
echo test finished, first few diffs are:
diff -y testShearTraj.shi testShearTraj.shi.ref | head
diff -q testShearTraj.shi testShearTraj.shi.ref
echo; echo

echo testing Kawasaki Ising
../src/shi.exe -n -ptestKawa.in -ctestKawaTraj.shi > testKawa.out
echo test finished, first few diffs are:
diff -y testKawaTraj.shi testKawaTraj.shi.ref | head
diff -q testKawaTraj.shi testKawaTraj.shi.ref | head
echo; echo

echo testing Asymmetric Exclusion Process
../src/shi.exe -n -ptestAsep.in -ctestAsepTraj.shi > testAsep.out
echo test finished, first few diffs are:
diff -y testAsepTraj.shi testAsepTraj.shi.ref | head
diff -q testAsepTraj.shi testAsepTraj.shi.ref | head
echo; echo

