#ifndef HAVE_RNG2
#define HAVE_RNG2



#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

class RNG2 {
 
  public:
    long  idum;
    void  init( long seed );
    void  srand2( long seed );
    float ran2();
    
  private:
    
    long idum2;
    long iy, *iv;
  
};



#undef IM1
#undef IM2
#undef AM
#undef IMM1
#undef IA1
#undef IA2
#undef IQ1
#undef IQ2
#undef IR1
#undef IR2
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX






#endif