/*****************************************************
**
**  S-PRES Copyright (c) 2010
**
**  Joshua T Berryman
**
**  This file is part of S-PRES
**
**  S-PRES is Free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  S-PRES is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
**  If you make use of this program for research 
**  resulting in a publication, please cite:
**
**  Joshua T Berryman and Tanja Schilling, 
**  Sampling rare events in nonequilibrium and nonstationary systems. 
**  
**  J Chem Phys, 133(24):244101, Dec 2010.
**
******************************************************/
#include <iostream>
#include <string>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <cstring>
#include "../../spres/src/spres.h"


///regurgitate inputs and print out bin boundaries
void spres::sanitiseInputs(simulation *state, int numPathsPerBin, long numSpresBins )
{

    double rcBinMax, rcBinMin;

    rcBinMax   = state->get_spresCriterionMax();
    rcBinMin   = state->get_spresCriterionMin();


    if( numPathsPerBin <= 0 || numSpresBins <= 0 )
    {
        cerr << "Error: something about these parameters does not make sense, bailing." << endl;
        cerr << numPathsPerBin << " " << numSpresBins  << endl;
        exit( EXIT_FAILURE );
    }

}





















